import argparse

from ._pycharm_remote_debugger import PycharmRemoteDebugger, CodeType


def get_args():
    parser = argparse.ArgumentParser(description="Pycharm remote debugger")
    parser.add_argument("-r", "--remote", help="Remote machine address", type=str)
    parser.add_argument("-p", "--port", help="Remote debugger port", type=int)
    parser.add_argument("-o", "--optional", help="Continue if not able to connect to the remote debugger", nargs='?',
                        default=False, type=bool, const=True)
    parser.add_argument("-t", "--timeout", help="Connection timeout", type=int,
                        default=PycharmRemoteDebugger.DEFAULT_CONNECTION_TIMEOUT)
    parser.add_argument("-m", "--module", help="Module to run", type=str, nargs="...")
    parser.add_argument("-f", "--file", help="Python script to run", type=str, nargs="...")
    args = parser.parse_args()

    if not args.port or not args.remote:
        raise ValueError("Missing remote IDE arguments - try: remote_debugger -r <remote_address> -p <port> ... ")

    if args.port is None and args.module is None:
        raise ValueError("Missing script or module to execute")

    return args


def main():
    args = get_args()
    code_type = CodeType.MODULE if args.module else CodeType.FILE
    debugger = PycharmRemoteDebugger(args.remote, args.port, code_type, args.module or args.file, args.optional)
    debugger.debug(args.timeout)


if __name__ == '__main__':
    main()



import logging
import sys


class CustomFormatter(logging.Formatter):
    grey = "\x1b[38;21m"
    yellow = "\x1b[33;21m"
    green = "\x1b[33;32m"
    red = "\x1b[31;21m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"

    FORMATS = {
        logging.DEBUG: grey + format + reset,
        logging.INFO: green + format + reset,
        logging.WARNING: yellow + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


log = logging.getLogger("pycharm_remote_debugger")
log.setLevel(logging.DEBUG)
_handler = logging.StreamHandler(sys.stdout)
_handler.setLevel(logging.DEBUG)
_fmt = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
_handler.setFormatter(CustomFormatter())

log.addHandler(_handler)



import io
import runpy
import socket
import sys
from contextlib import redirect_stderr
from enum import IntEnum
from typing import List

import pydevd_pycharm
import waiting

from ._logger import log


class CodeType(IntEnum):
    FILE = 0
    MODULE = 1


class PycharmRemoteDebugger:
    DEFAULT_CONNECTION_TIMEOUT = 30

    def __init__(self,
                 remote_machine: str,
                 port: int,
                 code_type: CodeType,
                 code_args: List[str] = None,
                 optional=False
                 ):

        self._remote_machine: str = remote_machine
        self._port: int = port
        self._code_type: CodeType = code_type
        self._code_args: List[str] = code_args
        self._optional: bool = optional

    def _debugger_login(self) -> bool:
        f = io.StringIO()

        with redirect_stderr(f):
            try:
                log.info(f"Trying to connect to the remote IDE {self._remote_machine}:{self._port}")
                pydevd_pycharm.settrace(self._remote_machine, port=self._port, stdoutToServer=True, stderrToServer=True)
                log.info(f"Debugger connected to IDE successfully")
                return True
            except ConnectionRefusedError as e:
                log.error(f"Error occurred while connecting to remote IDE, {e}")
                return False

    def _wait_for_debugger(self, timeout=DEFAULT_CONNECTION_TIMEOUT):
        try:
            waiting.wait(
                lambda: self._debugger_login(),
                timeout_seconds=timeout,
                sleep_seconds=1,
                waiting_for="remote debugger to be ready"
            )
        except socket.timeout:
            log.error(f"Timeout reached after {timeout} seconds")
            if not self._optional:
                raise

    def _run_file(self):
        runpy.run_path(self._code_args[0], run_name="__main__")

    def _run_module(self):
        try:
            run_module_as_main = runpy._run_module_as_main
        except AttributeError:
            log.warning("Can't find runpy._run_module_as_main module.")
            runpy.run_module(self._code_args[0], alter_sys=True)
        else:
            run_module_as_main(self._code_args[0], alter_argv=True)

    def debug(self, timeout: int):
        self._wait_for_debugger(timeout)
        argv = list(sys.argv)
        log.info(f"Running {self._code_type.name.lower()} {self._code_args[0]} with args {self._code_args[1:]}")

        if self._code_args is not None:
            if self._code_type == CodeType.MODULE:
                sys.argv = argv[argv.index("-m") + 1:]
                self._run_module()
            elif self._code_type == CodeType.FILE:
                sys.argv = argv[argv.index("-f") + 1:]
                self._run_file()

        sys.argv = argv
